__title__ = "Flask-Geo"
__description__ = "User Geolocation for Flask"
__url__ = "https://github.com/wankai/flask-geo"
__version_info__ = ("0", "1", "0")
__version__ = ".".join(__version_info__)
__author__ = "Wankai Zhang"
__author_email__ = "wankaizhang@gmail.com"
__maintainer__ = "Wankai Zhang"
__license__ = "MIT"
__copyright__ = "(c) 2022 by Wankai Zhang"
